Improve this Doc View Source

A factory which creates a resource object that lets you interact with RESTful server-side data sources.

The returned resource object has action methods which provide high-level behaviors without the need to interact with the low level $http service.

Requires the `ngResource` module to be installed.

By default, trailing slashes will be stripped from the calculated URLs, which can pose problems with server backends that do not expect that behavior. This can be disabled by configuring the `$resourceProvider` like this:

    app.config(['$resourceProvider', function($resourceProvider) {
      // Don't strip trailing slashes from calculated URLs
      $resourceProvider.defaults.stripTrailingSlashes = false;
    }]);

## Dependencies

- `$http`
- `$log`
- `$q`
- `$timeout`

## Usage

`$resource(url, [paramDefaults], [actions], options);`

### Arguments

Param | Type | Details
--- | --- | ---
url | string | A parameterized URL template with parameters prefixed by `:` as in `/user/:username`. If you are using a URL with a port number (e.g. `http://example.com:8080/api`), it will be respected. If you are using a url with a suffix, just add the suffix, like this: `$resource('http://example.com/resource.json')` or `$resource('http://example.com/:id.json')` or even `$resource('http://example.com/resource/:resource_id.:format')` If the parameter before the suffix is empty, :resource_id in this case, then the `/.` will be collapsed down to a single `.`. If you need this sequence to appear and not collapse then you can escape it with `/\.`.
paramDefaults (optional) | Object | Default values for `url` parameters. These can be overridden in `actions` methods. If a parameter value is a function, it will be called every time a param value needs to be obtained for a request (unless the param was overridden). The function will be passed the current data value as an argument. Each key value in the parameter object is first bound to url template if present and then any excess keys are appended to the url search query after the `?`. Given a template `/path/:verb` and parameter `{verb:'greet', salutation:'Hello'}` results in URL `/path/greet?salutation=Hello`. If the parameter value is prefixed with `@`, then the value for that parameter will be extracted from the corresponding property on the `data` object (provided when calling actions with a request body). For example, if the `defaultParam` object is `{someParam: '@someProp'}` then the value of `someParam` will be `data.someProp`. Note that the parameter will be ignored, when calling a "GET" action method (i.e. an action method that does not accept a request body)
actions (optional) | Object.<Object>= | Hash with declaration of custom actions that will be available in addition to the default set of resource actions (see below). If a custom action has the same key as a default action (e.g. `save`), then the default action will be overwritten, and not extended. The declaration should be created in the format of $http.config:
    {action1: {method:?, params:?, isArray:?, headers:?, ...},
     action2: {method:?, params:?, isArray:?, headers:?, ...},
     ...}
Where: `action` – {string} – The name of action. This name becomes the name of the method on your resource object. `method` – {string} – Case insensitive HTTP method (e.g. `GET`, `POST`, `PUT`, `DELETE`, `JSONP`, etc). `params` – {Object=} – Optional set of pre-bound parameters for this action. If any of the parameter value is a function, it will be called every time when a param value needs to be obtained for a request (unless the param was overridden). The function will be passed the current data value as an argument. `url` – {string} – action specific `url` override. The url templating is supported just like for the resource-level urls. `isArray` – {boolean=} – If true then the returned object for this action is an array, see `returns` section. `transformRequest` – `{function(data, headersGetter)|Array.<function(data, headersGetter)>}` – transform function or an array of such functions. The transform function takes the http request body and headers and returns its transformed (typically serialized) version. By default, transformRequest will contain one function that checks if the request data is an object and serializes it using `angular.toJson`. To prevent this behavior, set `transformRequest` to an empty array: `transformRequest: []` `transformResponse` – `{function(data, headersGetter, status)|Array.<function(data, headersGetter, status)>}` – transform function or an array of such functions. The transform function takes the http response body, headers and status and returns its transformed (typically deserialized) version. By default, transformResponse will contain one function that checks if the response looks like a JSON string and deserializes it using `angular.fromJson`. To prevent this behavior, set `transformResponse` to an empty array: `transformResponse: []` `cache` – `{boolean|Cache}` – If true, a default $http cache will be used to cache the GET request, otherwise if a cache instance built with $cacheFactory is supplied, this cache will be used for caching. `timeout` – `{number}` – timeout in milliseconds. Note: In contrast to $http.config, promises are not supported in $resource, because the same value would be used for multiple requests. If you are looking for a way to cancel requests, you should use the `cancellable` option. `cancellable` – `{boolean}` – if set to true, the request made by a "non-instance" call will be cancelled (if not already completed) by calling `$cancelRequest()` on the call's return value. Calling `$cancelRequest()` for a non-cancellable or an already completed/cancelled request will have no effect. `withCredentials` - `{boolean}` - whether to set the `withCredentials` flag on the XHR object. See requests with credentials for more information. `responseType` - `{string}` - see requestType. `interceptor` - `{Object=}` - The interceptor object has two optional methods - `response` and `responseError`. Both `response` and `responseError` interceptors get called with `http response` object. See $http interceptors. In addition, the resource instance or array object is accessible by the `resource` property of the `http response` object. Keep in mind that the associated promise will be resolved with the value returned by the response interceptor, if one is specified. The default response interceptor returns `response.resource` (i.e. the resource instance or array). `hasBody` - `{boolean}` - allows to specify if a request body should be included or not. If not specified only POST, PUT and PATCH requests will have a body.
options | Object | Hash with custom settings that should extend the default `$resourceProvider` behavior. The supported options are: `stripTrailingSlashes` – {boolean} – If true then the trailing slashes from any calculated URL will be stripped. (Defaults to true.) `cancellable` – {boolean} – If true, the request made by a "non-instance" call will be cancelled (if not already completed) by calling `$cancelRequest()` on the call's return value. This can be overwritten per action. (Defaults to false.)

### Returns

Object | A resource "class" object with methods for the default set of resource actions optionally extended with custom `actions`. The default set contains these actions:
    { 'get':    {method:'GET'},
      'save':   {method:'POST'},
      'query':  {method:'GET', isArray:true},
      'remove': {method:'DELETE'},
      'delete': {method:'DELETE'} };
Calling these methods invoke an `$http` with the specified http method, destination and parameters. When the data is returned from the server then the object is an instance of the resource class. The actions `save`, `remove` and `delete` are available on it as methods with the `$` prefix. This allows you to easily perform CRUD operations (create, read, update, delete) on server-side data like this:
    var User = $resource('/user/:userId', {userId:'@id'});
    var user = User.get({userId:123}, function() {
      user.abc = true;
      user.$save();
    });
It is important to realize that invoking a $resource object method immediately returns an empty reference (object or array depending on `isArray`). Once the data is returned from the server the existing reference is populated with the actual data. This is a useful trick since usually the resource is assigned to a model which is then rendered by the view. Having an empty object results in no rendering, once the data arrives from the server then the object is populated with the data and the view automatically re-renders itself showing the new data. This means that in most cases one never has to write a callback function for the action methods. The action methods on the class object or instance object can be invoked with the following parameters: "class" actions without a body: `Resource.action([parameters], [success], [error])` "class" actions with a body: `Resource.action([parameters], postData, [success], [error])` instance actions: `instance.$action([parameters], [success], [error])` When calling instance methods, the instance itself is used as the request body (if the action should have a body). By default, only actions using `POST`, `PUT` or `PATCH` have request bodies, but you can use the `hasBody` configuration option to specify whether an action should have a body or not (regardless of its HTTP method). Success callback is called with (value (Object|Array), responseHeaders (Function), status (number), statusText (string)) arguments, where the value is the populated resource instance or collection object. The error callback is called with (httpResponse) argument. Class actions return empty instance (with additional properties below). Instance actions return promise of the action. The Resource instances and collections have these additional properties: `$promise`: the promise of the original server interaction that created this instance or collection. On success, the promise is resolved with the same resource instance or collection object, updated with data from server. This makes it easy to use in resolve section of $routeProvider.when() to defer view rendering until the resource(s) are loaded. On failure, the promise is rejected with the http response object. If an interceptor object was provided, the promise will instead be resolved with the value returned by the interceptor. `$resolved`: `true` after first server interaction is completed (either with success or rejection), `false` before that. Knowing if the Resource has been resolved is useful in data-binding. The Resource instances and collections have these additional methods: `$cancelRequest`: If there is a cancellable, pending request related to the instance or collection, calling this method will abort the request. The Resource instances have these additional methods: `toJSON`: It returns a simple object without any of the extra properties added as part of the Resource API. This object can be serialized through `angular.toJson` safely without attaching Angular-specific fields. Notice that `JSON.stringify` (and `angular.toJson`) automatically use this method when serializing a Resource instance (see MDN).

## Examples

# Credit card resource

    // Define CreditCard class
    var CreditCard = $resource('/user/:userId/card/:cardId',
     {userId:123, cardId:'@id'}, {
      charge: {method:'POST', params:{charge:true}}
     });

    // We can retrieve a collection from the server
    var cards = CreditCard.query(function() {
      // GET: /user/123/card
      // server returns: [ {id:456, number:'1234', name:'Smith'} ];

      var card = cards[0];
      // each item is an instance of CreditCard
      expect(card instanceof CreditCard).toEqual(true);
      card.name = "J. Smith";
      // non GET methods are mapped onto the instances
      card.$save();
      // POST: /user/123/card/456 {id:456, number:'1234', name:'J. Smith'}
      // server returns: {id:456, number:'1234', name: 'J. Smith'};

      // our custom method is mapped as well.
      card.$charge({amount:9.99});
      // POST: /user/123/card/456?amount=9.99&charge=true {id:456, number:'1234', name:'J. Smith'}
    });

    // we can create an instance as well
    var newCard = new CreditCard({number:'0123'});
    newCard.name = "Mike Smith";
    newCard.$save();
    // POST: /user/123/card {number:'0123', name:'Mike Smith'}
    // server returns: {id:789, number:'0123', name: 'Mike Smith'};
    expect(newCard.id).toEqual(789);

The object returned from this function execution is a resource "class" which has "static" method for each action in the definition.

Calling these methods invoke `$http` on the `url` template with the given `method`, `params` and `headers`.

# User resource

When the data is returned from the server then the object is an instance of the resource type and all of the non-GET methods are available with `$` prefix. This allows you to easily support CRUD operations (create, read, update, delete) on server-side data.

    var User = $resource('/user/:userId', {userId:'@id'});
    User.get({userId:123}, function(user) {
      user.abc = true;
      user.$save();
    });

It's worth noting that the success callback for `get`, `query` and other methods gets passed in the response that came from the server as well as $http header getter function, so one could rewrite the above example and get access to http headers as:

    var User = $resource('/user/:userId', {userId:'@id'});
    User.get({userId:123}, function(user, getResponseHeaders){
      user.abc = true;
      user.$save(function(user, putResponseHeaders) {
        //user => saved user object
        //putResponseHeaders => $http header getter
      });
    });

You can also access the raw `$http` promise via the `$promise` property on the object returned

    var User = $resource('/user/:userId', {userId:'@id'});
    User.get({userId:123})
        .$promise.then(function(user) {
          $scope.user = user;
        });

# Creating a custom 'PUT' request

In this example we create a custom method on our resource to make a PUT request

    var app = angular.module('app', ['ngResource', 'ngRoute']);

    // Some APIs expect a PUT request in the format URL/object/ID
    // Here we are creating an 'update' method
    app.factory('Notes', ['$resource', function($resource) {
    return $resource('/notes/:id', null,
        {
            'update': { method:'PUT' }
        });
    }]);

    // In our controller we get the ID from the URL using ngRoute and $routeParams
    // We pass in $routeParams and our Notes factory along with $scope
    app.controller('NotesCtrl', ['$scope', '$routeParams', 'Notes',
                                       function($scope, $routeParams, Notes) {
    // First get a note object from the factory
    var note = Notes.get({ id:$routeParams.id });
    $id = note.id;

    // Now call update passing in the ID first then the object you are updating
    Notes.update({ id:$id }, note);

    // This will PUT /notes/ID with the note object in the request payload
    }]);

# Cancelling requests

If an action's configuration specifies that it is cancellable, you can cancel the request related to an instance or collection (as long as it is a result of a "non-instance" call):

    // ...defining the `Hotel` resource...
    var Hotel = $resource('/api/hotel/:id', {id: '@id'}, {
      // Let's make the `query()` method cancellable
      query: {method: 'get', isArray: true, cancellable: true}
    });

    // ...somewhere in the PlanVacationController...
    ...
    this.onDestinationChanged = function onDestinationChanged(destination) {
      // We don't care about any pending request for hotels
      // in a different destination any more
      this.availableHotels.$cancelRequest();

      // Let's query for hotels in '<destination>'
      // (calls: /api/hotel?location=<destination>)
      this.availableHotels = Hotel.query({location: destination});
    };